@@ -16,7 +16,7 @@ from abc import ABC
 from typing import List
 
 
-class ConsumerConfig(ABC):  # pylint: disable=too-few-public-methods
+class ConsumerConfig(ABC):
     """Some of the confluence exposed consumer properties"""
 
     BOOTSTRAP_SERVERS = 'bootstrap.servers'
