@@ -20,32 +20,40 @@ from typing import Any
 from hspylib.core.enums.enumeration import Enumeration
 
 
-class _JsonType(Enumeration):
+class JsonType(Enumeration):
     """TODO"""
 
     # @formatter:off
-    BOOLEAN         = 'boolean'
-    INTEGER         = 'integer'  # int|long
-    NUMBER          = 'number'   # float|double
     STRING          = 'string'   # string|bytes|enum|fixed
+    NUMBER          = 'number'   # float|double
+    INTEGER         = 'integer'  # int|long
     OBJECT          = 'object'   # record|map
     ARRAY           = 'array'    # array
+    BOOLEAN         = 'boolean'  # bool
+    ENUM            = 'enum'     # array or string enumeration
     # @formatter:on
 
     def empty_value(self) -> Any:
         """TODO"""
+
         if self.value == 'boolean':
-            return False
+            value = False
         elif self.value == 'integer':
-            return 0
+            value = 0
         elif self.value == 'number':
-            return 0.0
+            value = 0.0
         elif self.value == 'object':
-            return defaultdict()
+            value = defaultdict()
         elif self.value == 'array':
-            return []
+            value = []
         else:
-            return ''
+            value = ''
+
+        return value
+
+    def is_primitive(self) -> bool:
+        return self.value not in ['object', 'array']
 
     def is_object(self):
+        """TODO"""
         return self.value == 'object'
