@@ -1,14 +1,14 @@
 # _*_ coding: utf-8 _*_
 #
-# hspylib-kafman v0.9.32
+# hspylib-kafman v0.9.65
 #
 # Package: main.kafman.views
 """Package initialization."""
 
 __all__ = [
-  'form_area', 
-  'form_pane', 
+  'dialogs', 
   'indexes', 
-  'main_qt_view'
+  'main_qt_view', 
+  'promotions'
 ]
-__version__ = '0.9.32'
+__version__ = '0.9.65'
