@@ -19,26 +19,29 @@ import logging as log
 import os
 import re
 from collections import defaultdict
+from copy import deepcopy
 from json.decoder import JSONDecodeError
 from pathlib import Path
 from typing import List, Optional, Tuple, Union
 
 from confluent_kafka.cimpl import Consumer
-from fastavro.validation import validate as validate_schema
+from hspylib.core.enums.charset import Charset
 from hspylib.core.exception.exceptions import InvalidInputError, InvalidStateError, UnsupportedSchemaError
-from hspylib.core.tools.commons import dirname, get_path, now, now_ms
+from hspylib.core.tools.commons import dirname, now, now_ms
 from hspylib.core.tools.preconditions import check_state
 from hspylib.core.tools.text_tools import strip_escapes, strip_extra_spaces, strip_linebreaks
 from hspylib.modules.cli.icons.font_awesome.dashboard_icons import DashboardIcons
 from hspylib.modules.cli.icons.font_awesome.form_icons import FormIcons
+from hspylib.modules.fetch.fetch import is_reachable
 from hspylib.modules.qt.promotions.hstacked_widget import HStackedWidget
 from hspylib.modules.qt.promotions.htablemodel import HTableModel
 from hspylib.modules.qt.stream_capturer import StreamCapturer
 from hspylib.modules.qt.views.qt_view import QtView
 from PyQt5.QtCore import Qt
-from PyQt5.QtGui import QColor, QFont
-from PyQt5.QtWidgets import QFileDialog
+from PyQt5.QtGui import QColor
+from PyQt5.QtWidgets import QFileDialog, QMessageBox
 
+from kafman.__classpath__ import Classpath, get_source
 from kafman.core.constants import MAX_HISTORY_SIZE_BYTES, StatusColor
 from kafman.core.consumer.consumer_config import ConsumerConfig
 from kafman.core.consumer.consumer_worker import ConsumerWorker
@@ -53,25 +56,23 @@ from kafman.core.schema.registry_subject import RegistrySubject
 from kafman.core.schema.schema_factory import SchemaFactory
 from kafman.core.schema.schema_registry import SchemaRegistry
 from kafman.core.statistics_worker import StatisticsWorker
-from kafman.views.filters_dialog import FiltersDialog
+from kafman.views.dialogs.filters_dialog import FiltersDialog
+from kafman.views.dialogs.settings_dialog import SettingsDialog
 from kafman.views.indexes import StkProducerEdit, StkTools, Tabs
-from kafman.views.settings_dialog import SettingsDialog
-
-HERE = get_path(__file__)
 
 
 class MainQtView(QtView):
     """Main application view"""
-    VERSION = (HERE / "../.version").read_text()
+    VERSION = get_source(".version").read_text(encoding=Charset.UTF_8.value)
+
+    SCHEMA_DIR = (Classpath.RESOURCE_DIR / "schema")
+
+    FORMS_DIR = (Classpath.RESOURCE_DIR / "forms")
 
     HISTORY_FILE = f"{os.getenv('HOME', os.getcwd())}/.kafman-history.properties"
 
-    SCHEMA_DIR = str(HERE / "../resources/schema")
-
-    FORMS_DIR = str(HERE / "../resources/forms")
-
     KAFKA_INTERNAL_TOPICS = (
-        '_confluent-', '_schemas', '__consumer_offsets', '__transaction_state'
+        '_confluent', '_schemas', '__consumer_offsets', '__transaction_state'
     )
 
     @staticmethod
@@ -87,12 +88,8 @@ class MainQtView(QtView):
         """Whether the provided text is a json code or not"""
         if not text:
             return False
-
         t = text.strip()
-        # @formatter:off
-        return (t.startswith('{') and t.endswith('}')) \
-                or (t.startswith('[') and t.endswith(']'))
-        # @formatter:on
+        return (t.startswith('{') and t.endswith('}')) or (t.startswith('[') and t.endswith(']'))
 
     def __init__(self):
         # Must come after the initialization above
@@ -126,7 +123,6 @@ class MainQtView(QtView):
 
     def _setup_ui(self) -> None:
         """Setup UI: Connect signals and Setup components"""
-        self.set_default_font(QFont("DroidSansMono Nerd Font", 14))
         self.window.setWindowTitle(f"Kafman v{self.VERSION}")
         self.window.resize(1024, 768)
         self._setup_general_controls()
@@ -141,7 +137,7 @@ class MainQtView(QtView):
         self.ui.lbl_status_text.setTextFormat(Qt.RichText)
         self.ui.lbl_status_text.set_elidable(True)
         self.ui.tbtn_test_registry_url.setText(FormIcons.CHECK_CIRCLE.value)
-        self.ui.tbtn_test_registry_url.clicked.connect(self._test_registry_url)
+        self.ui.tbtn_test_registry_url.clicked.connect(lambda: self._test_registry_url(pop_warn_box=True))
         self.ui.tbtn_sel_schema.setText(DashboardIcons.FOLDER_OPEN.value)
         self.ui.tbtn_sel_schema.clicked.connect(self._add_schema)
         self.ui.tbtn_desel_schema.setText(FormIcons.CLEAR.value)
@@ -172,7 +168,7 @@ class MainQtView(QtView):
         self.ui.tbtn_produce.setText(DashboardIcons.SEND.value)
         self.ui.tbtn_prod_clear_topics.setText(FormIcons.CLEAR.value)
         self.ui.tbtn_prod_find_topics.setText(DashboardIcons.SEARCH.value)
-        self.ui.tbtn_prod_find_topics.clicked.connect(lambda: self._find_broker_topics(is_producer=True))
+        self.ui.tbtn_prod_find_topics.clicked.connect(lambda: self._fetch_broker_topics(is_producer=True))
         self.ui.tbtn_prod_add_topics.setText(FormIcons.PLUS.value)
         self.ui.tbtn_prod_add_topics.clicked.connect(self._add_topic)
         self.ui.tbtn_prod_del_topics.setText(FormIcons.MINUS.value)
@@ -213,13 +209,15 @@ class MainQtView(QtView):
         self.ui.tbtn_cons_connect.setStyleSheet('QToolButton {color: #2380FA;}')
         self.ui.tbtn_cons_clear_topics.setText(FormIcons.CLEAR.value)
         self.ui.tbtn_cons_find_topics.setText(DashboardIcons.SEARCH.value)
-        self.ui.tbtn_cons_find_topics.clicked.connect(lambda: self._find_broker_topics(is_producer=False))
+        self.ui.tbtn_cons_find_topics.clicked.connect(lambda: self._fetch_broker_topics(is_producer=False))
         self.ui.tbtn_cons_add_topics.setText(FormIcons.PLUS.value)
         self.ui.tbtn_cons_add_topics.clicked.connect(lambda: self._add_topic(is_producer=False))
         self.ui.tbtn_cons_del_topics.setText(FormIcons.MINUS.value)
         self.ui.tbtn_cons_del_topics.clicked.connect(lambda: self._del_topic(is_producer=False))
         self.ui.tbtn_cons_filters.setText(FormIcons.FILTER.value)
         self.ui.tbtn_cons_filters.clicked.connect(self._add_consumer_filters)
+        self.ui.tbtn_cons_filters_refresh.setText(FormIcons.REFRESH.value)
+        self.ui.tbtn_cons_filters_refresh.clicked.connect(self.ui.tbl_consumer.refresh)
         self.ui.lst_cons_settings.currentRowChanged.connect(self._get_setting)
         self.ui.lst_cons_settings.set_editable()
         self.ui.lst_cons_settings.itemChanged.connect(self._edit_setting)
@@ -259,8 +257,11 @@ class MainQtView(QtView):
 
     def _schema(self) -> KafkaSchema:
         """Return the selected serialization schema"""
-        sel_schema = self.ui.cmb_sel_schema.currentText()
-        return self._all_schemas[sel_schema] if sel_schema in self._all_schemas else PlainSchema()
+        schema = self._producer.schema() if self._is_producer() else self._consumer.schema()
+        if not schema:
+            sel_schema = self.ui.cmb_sel_schema.currentText()
+            schema = self._all_schemas[sel_schema] if sel_schema in self._all_schemas else PlainSchema()
+        return schema
 
     def _messages(self) -> Union[str, List[str]]:
         """Return the selected messages or build a json message from form"""
@@ -268,7 +269,7 @@ class MainQtView(QtView):
         if isinstance(schema, PlainSchema) or self.ui.stk_producer_edit.currentIndex() == StkProducerEdit.TEXT.value:
             text = self.ui.txt_producer.toPlainText()
             msgs = [text] if self._is_json(text) else text.split('\n')
-            return list(map(lambda x: strip_linebreaks(x), filter(None, msgs)))
+            return list(map(strip_linebreaks, filter(None, msgs)))
 
         return strip_extra_spaces(strip_linebreaks(self._form_to_message()))
 
@@ -287,7 +288,7 @@ class MainQtView(QtView):
     def _add_consumer_filters(self):
         filters_dlg = FiltersDialog(self.ui.splitter_pane, self.ui.tbl_consumer.filters())
         filters_dlg.filtersChanged.connect(
-            lambda s: self.ui.lbl_cons_filters.setText(s))
+            lambda s: self.ui.lbl_cons_filters.setText(f'Filters: {s}'))
         filters_dlg.show()
 
     def _open_message_file(self) -> None:
@@ -296,7 +297,7 @@ class MainQtView(QtView):
             'Open file', self._last_used_dir or '.', "Select a message file (*.txt *.json *.csv)")
         if file_tuple and file_tuple[0]:
             self._last_used_dir = dirname(file_tuple[0][0])
-            self.ui.txt_producer.set_plain_text(Path(file_tuple[0][0]).read_text())
+            self.ui.txt_producer.set_plain_text(Path(file_tuple[0][0]).read_text(encoding=Charset.UTF_8.value))
 
     def _save_message_file(self) -> None:
         file_tuple = QFileDialog.getSaveFileName(
@@ -304,7 +305,7 @@ class MainQtView(QtView):
             'Save file', self._last_used_dir or '.', "Save file as (*.*)")
         if file_tuple and file_tuple[0]:
             self._last_used_dir = dirname(file_tuple[0])
-            with open(Path(file_tuple[0]), "w") as fd_file:
+            with open(Path(file_tuple[0]), "w", encoding=Charset.UTF_8.value) as fd_file:
                 fd_file.write(self.ui.txt_producer.toPlainText())
 
     def _form_to_message(self, validate: bool = True) -> str:
@@ -327,7 +328,7 @@ class MainQtView(QtView):
         text = text.replace("'", '"')
         if self._is_json(text):
             try:
-                formatted = json.dumps(json.loads(text), indent=2, sort_keys=False)
+                formatted = json.dumps(json.loads(text), indent=2)
                 self.ui.txt_producer.set_plain_text(formatted)
                 self.ui.tbtn_format_msg.setStyleSheet("QToolButton {color: #28C941;}")
                 self._display_text('The provided json is valid', StatusColor.green)
@@ -343,10 +344,10 @@ class MainQtView(QtView):
 
     def _validate_schema_form(self):
         """TODO"""
-        schema = self._schema().get_content_dict()
-        json_form = json.loads(self.ui.scr_schema_fields.values())
+        schema = self._schema()
         try:
-            validate_schema(json_form, schema, raise_errors=True)
+            json_form = json.loads(self.ui.scr_schema_fields.values())
+            schema.validate(json_form)
             self._display_text('Schema form is valid', StatusColor.green)
         except Exception as err:
             self._display_error(f'Schema form is not valid:: {err}')
@@ -380,7 +381,7 @@ class MainQtView(QtView):
                         self._display_error(f"Unsupported schema: => {str(err)}")
                     except InvalidStateError as err:
                         self._display_error(f"Add schema failed: => {str(err)}")
-                        raise InvalidStateError(err)
+                        raise InvalidStateError(err) from err
 
     def _deselect_schema(self):
         """Deselect current selected serialization schema"""
@@ -392,11 +393,8 @@ class MainQtView(QtView):
         if schema_name:
             content = self._all_schemas[schema_name].get_content_dict()
             self.ui.tool_box.setCurrentIndex(StkTools.SCHEMAS.value)
-            self.ui.txt_sel_schema.set_plain_text(json.dumps(content, indent=2, sort_keys=False))
-            if self._build_schema_layout():
-                self.ui.stk_producer_edit.setCurrentIndex(StkProducerEdit.FORM.value)
-                self.ui.tbtn_form_view.setEnabled(True)
-                self.ui.lbl_schema_fields.setText(f"{schema_name} Schema Fields")
+            self.ui.txt_sel_schema.set_plain_text(json.dumps(content, indent=1))
+            self.ui.tbtn_form_view.setEnabled(True)
         else:
             self.ui.stk_producer_edit.setCurrentIndex(StkProducerEdit.TEXT.value)
             self.ui.txt_sel_schema.set_plain_text('')
@@ -406,23 +404,22 @@ class MainQtView(QtView):
 
     def _build_schema_layout(self) -> bool:
         """Build a form based on the selected schema using a grid layout"""
-        sel_schema = self.ui.cmb_sel_schema.currentText()
-        if sel_schema:
-            schema = self._all_schemas[sel_schema]
+        schema = self._producer.schema()
+        if schema:
             try:
-                if schema:
-                    self._cleanup_schema_layout()
-                    form_widget = HStackedWidget(self.ui.tab_widget)
-                    form_widget.currentChanged.connect(self._schema_form_changed)
-                    form_widget.installEventFilter(form_widget)
-                    schema.create_schema_form_widget(form_widget)
-                    self.ui.scr_schema_fields.setWidget(form_widget)
-                    self._schema_form_changed(0)
-                    return True
+                self._cleanup_schema_layout()
+                form_widget = HStackedWidget(self.ui.tab_widget)
+                form_widget.currentChanged.connect(self._schema_form_changed)
+                form_widget.installEventFilter(form_widget)
+                schema.create_schema_form_widget(form_widget)
+                self.ui.scr_schema_fields.setWidget(form_widget)
+                self._schema_form_changed(0)
+                self.ui.lbl_schema_fields.setText(f"{schema.get_schema_name()} Schema Fields")
+                return True
             except AttributeError as err:
                 self.ui.cmb_sel_schema.removeItem(self.ui.cmb_sel_schema.currentIndex())
                 self._display_error(f"Invalid schema {schema} was not loaded => {err}")
-                raise InvalidStateError(err)
+                raise InvalidStateError(err) from err
         return False
 
     def _cleanup_schema_layout(self) -> None:
@@ -496,17 +493,22 @@ class MainQtView(QtView):
             else:
                 self._display_error(f"Setting {setting} is required")
 
-    def _find_broker_topics(self, is_producer: bool) -> None:
+    def _fetch_broker_topics(self, is_producer: bool) -> None:
         """Find all topics from broker"""
         config = {
-            k: v for (k, v) in self._all_settings['consumer'].items() if k in ['bootstrap.servers', 'group.id']
+            k: v for (k, v) in self._all_settings['consumer'].items()
+            if k in [ConsumerConfig.BOOTSTRAP_SERVERS, ConsumerConfig.GROUP_ID]
         }
+        brokers = config[ConsumerConfig.BOOTSTRAP_SERVERS]
+        if not is_reachable(brokers):
+            self._display_error(f"Unable to connect to kafka brokers: [{brokers}]", pop_warn_box = True)
+            return
         consumer = Consumer(config)
         topics = [
             k for (k, v) in consumer.list_topics().topics.items()
-            if not k.startswith(self.KAFKA_INTERNAL_TOPICS) and not k.startswith(f"{config['group.id']}--")
+            if not k.startswith(self.KAFKA_INTERNAL_TOPICS) and not k.startswith(f"{config[ConsumerConfig.GROUP_ID]}--")
         ]
-        self._display_text(f"Retrieved {len(topics)} topics from broker: {config['bootstrap.servers']}")
+        self._display_text(f"Retrieved {len(topics)} topics from broker: {brokers}")
         consumer.close()
         for topic in topics:
             self._add_topic(topic, is_producer)
@@ -543,7 +545,7 @@ class MainQtView(QtView):
         self.ui.tbtn_test_registry_url.setText(FormIcons.CHECK_CIRCLE.value)
         self._registry.invalidate()
 
-    def _test_registry_url(self, skip_if_tested: bool = True) -> Optional[str]:
+    def _test_registry_url(self, skip_if_tested: bool = True, pop_warn_box: bool = False) -> Optional[str]:
         """Check is the provided schema registry url is valid or not"""
         url = self.ui.cmb_registry_url.currentText()
         if self._registry.is_valid() and skip_if_tested:
@@ -562,7 +564,7 @@ class MainQtView(QtView):
             else:
                 self.ui.tbtn_test_registry_url.setText(FormIcons.ERROR.value)
                 self.ui.tbtn_test_registry_url.setStyleSheet("QToolButton {color: #FF554D;}")
-                self._display_error(f"Host {url} is unreachable")
+                self._display_error(f"Unable to connect to registry server: [{url}]", pop_warn_box=pop_warn_box)
         else:
             self.ui.tbtn_test_registry_url.setText(FormIcons.UNCHECK_CIRCLE.value)
             self.ui.tbtn_test_registry_url.setStyleSheet("QToolButton {color: #FF554D;}")
@@ -576,7 +578,7 @@ class MainQtView(QtView):
         sel_model = table.selectionModel()
         if sel_model:
             index_list = sel_model.selectedIndexes()
-            offset_idx, header = next(
+            offset_idx, _ = next(
                 ((i, h) for i, h in enumerate(model.headers) if str(h).lower() == 'offset'), None)
             offset = -1
             for index in index_list:
@@ -589,21 +591,31 @@ class MainQtView(QtView):
     def _toggle_start_producer(self) -> None:
         """Start/Stop the producer."""
         started = self._producer.is_started()
-        settings = self._settings()
         if started:  # Stop
             self.ui.tool_box.setCurrentIndex(StkTools.SETTINGS.value)
             self._producer.stop_producer()
             self._display_text("Production to kafka topics stopped", StatusColor.yellow)
         else:  # Start
             topics = self._topics(True)
-            if topics:
-                schema_name = f" using schema \"{str(self._schema())}\"" if self._schema() else ''
-                self.ui.tool_box.setCurrentIndex(StkTools.STATISTICS.value)
-                self._producer.start_producer(settings, self._schema())
-                self._display_text(f"Started producing to topics {topics}{schema_name}", StatusColor.green)
-            else:
-                self._display_error('Must subscribe to at least one topic')
+            if not topics:
+                self._display_error('Must subscribe to at least one topic', pop_warn_box=True)
                 return
+            schema = self._schema()
+            settings = deepcopy(self._settings())
+            settings.update(schema.settings())
+            settings = {k: v for k, v in settings.items() if not k.endswith('.deserializer')}
+            brokers = settings[ProducerConfig.BOOTSTRAP_SERVERS]
+            if not is_reachable(brokers):
+                self._display_error(f"Unable to connect to kafka brokers: [{brokers}]", pop_warn_box=True)
+                return
+            schema_name = f" using schema \"{str(self._schema())}\"" if self._schema() else ''
+            self.ui.tool_box.setCurrentIndex(StkTools.STATISTICS.value)
+            self._producer.start_producer(settings, schema)
+            if self._build_schema_layout():
+                self.ui.stk_producer_edit.setCurrentIndex(StkProducerEdit.FORM.value)
+            self._display_text(f"Started producer. Topics: {topics}  Schema: {schema_name}", StatusColor.green)
+            log.info(f'Producer settings: {settings}')
+
         self.ui.tbtn_format_msg.setEnabled(not started)
         self.ui.tbtn_export_form.setEnabled(not started)
         self.ui.tbtn_validate_form.setEnabled(not started)
@@ -624,22 +636,30 @@ class MainQtView(QtView):
     def _toggle_start_consumer(self) -> None:
         """Start/Stop the consumer."""
         started = self._consumer.is_started()
-        settings = self._settings()
         if started:  # Stop
-            self.ui.tool_box.setCurrentIndex(StkTools.STATISTICS.value)
+            self.ui.tool_box.setCurrentIndex(StkTools.SETTINGS.value)
             self._consumer.stop_consumer()
             self._display_text("Consumption from kafka topics stopped", StatusColor.yellow)
         else:  # Start
             topics = self._topics(False)
-            if topics:
-                schema_name = f" using schema \"{str(self._schema())}\"" if self._schema() else ''
-                self.ui.tool_box.setCurrentIndex(StkTools.SCHEMAS.value)
-                self._consumer.start_consumer(settings, self._schema())
-                self._consumer.consume(topics)
-                self._display_text(f"Started consuming from topics {topics}{schema_name}", StatusColor.green)
-            else:
-                self._display_error('Must subscribe to at least one topic')
+            if not topics:
+                self._display_error('Must subscribe to at least one topic', pop_warn_box=True)
                 return
+            schema = self._schema()
+            settings = deepcopy(self._settings())
+            settings.update(schema.settings())
+            settings = {k: v for k, v in settings.items() if not k.endswith('.serializer')}
+            brokers = settings[ProducerConfig.BOOTSTRAP_SERVERS]
+            if not is_reachable(brokers):
+                self._display_error(f"Unable to connect to kafka brokers: [{brokers}]", pop_warn_box=True)
+                return
+            schema_name = f" using schema \"{str(self._schema())}\"" if self._schema() else ''
+            self.ui.tool_box.setCurrentIndex(StkTools.STATISTICS.value)
+            self._consumer.start_consumer(settings, schema)
+            self._consumer.consume(topics)
+            self._display_text(f"Started consumer. Topics {topics}  Schema: {schema_name}", StatusColor.green, )
+            log.info(f'Consumer settings: {settings}')
+
         self.ui.cmb_cons_topics.setEnabled(started)
         self.ui.tbtn_cons_add_topics.setEnabled(started)
         self.ui.tbtn_cons_del_topics.setEnabled(started)
@@ -659,16 +679,24 @@ class MainQtView(QtView):
         except InvalidInputError as err:
             self._display_error(str(err))
 
-    def _display_error(self, message: str, add_console: bool = True) -> None:
+    def _display_error(self, message: str, console_print: bool = True, pop_warn_box: bool = False) -> None:
         """Display an error at the status bar (and console if required)"""
-        self._display_text(message, StatusColor.red, add_console)
 
-    def _display_text(self, message: str, color: QColor = None, add_console: bool = True) -> None:
+        self._display_text(message, StatusColor.red, console_print, log_it=False)
+        log.error(message)
+        if pop_warn_box:
+            QMessageBox.warning(self.ui.tab_widget, "Attention", message, QMessageBox.Ok)
+
+    def _display_text(
+        self, message: str, color: QColor = None, console_print: bool = True, log_it: bool = True) -> None:
         """Display a text at the status bar (and console if required)"""
+
         message = f"<font color='{color.name() if color else 'white'}'>{message}</font>"
         self.ui.lbl_status_text.setText(message)
-        if add_console:
+        if console_print:
             self._console_print(f"-> {message}", color)
+        if log_it:
+            log.info(message)
 
     def _update_stats(
         self,
@@ -679,6 +707,7 @@ class MainQtView(QtView):
         average_produced: int,
         average_consumed: int) -> None:
         """Update the consumer and producer statistics"""
+
         self.ui.lbl_stats_produced.setText(str(produced_total))
         self.ui.lbl_stats_produced_ps.setText(str(produced_in_a_tick))
         self.ui.lbl_stats_produced_avg.setText(str(average_produced))
